/*
 * Copyright (c) 2001-2002 Conexant Systems, Inc.
 * 
 * 1.   Permitted use. Redistribution and use in source and binary forms,
 * with or without modification, are permitted under the terms set forth
 * herein.
 * 
 * 2.   Disclaimer of Warranties. CONEXANT AND OTHER CONTRIBUTORS MAKE NO
 * REPRESENTATION ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.
 * IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTIES OF ANY KIND.
 * CONEXANT AND OTHER CONTRIBUTORS DISCLAIMS ALL WARRANTIES WITH REGARD TO
 * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, GOOD TITLE AND AGAINST INFRINGEMENT.
 * 
 * This software has not been formally tested, and there is no guarantee that
 * it is free of errors including, but not limited to, bugs, defects,
 * interrupted operation, or unexpected results. Any use of this software is
 * at user's own risk.
 * 
 * 3.   No Liability.
 * 
 * (a) Conexant or contributors shall not be responsible for any loss or
 * damage to Company, its customers, or any third parties for any reason
 * whatsoever, and CONEXANT OR CONTRIBUTORS SHALL NOT BE LIABLE FOR ANY
 * ACTUAL, DIRECT, INDIRECT, SPECIAL, PUNITIVE, INCIDENTAL, OR CONSEQUENTIAL
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED, WHETHER IN CONTRACT, STRICT OR OTHER LEGAL THEORY OF
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 * 
 * (b) User agrees to hold Conexant and contributors harmless from any
 * liability, loss, cost, damage or expense, including attorney's fees,
 * as a result of any claims which may be made by any person, including
 * but not limited to User, its agents and employees, its customers, or
 * any third parties that arise out of or result from the manufacture,
 * delivery, actual or alleged ownership, performance, use, operation
 * or possession of the software furnished hereunder, whether such claims
 * are based on negligence, breach of contract, absolute liability or any
 * other legal theory.
 * 
 * 4.   Notices. User hereby agrees not to remove, alter or destroy any
 * copyright, trademark, credits, other proprietary notices or confidential
 * legends placed upon, contained within or associated with the Software,
 * and shall include all such unaltered copyright, trademark, credits,
 * other proprietary notices or confidential legends on or in every copy of
 * the Software.
 * 
 */

//#define __KERNEL_SYSCALLS__

#include "oscompat.h"
#include "ostypedefs.h"
#include "osservices.h"
#include "oslinux.h"
#include "osmemory.h"
#include "ostime.h"
#include "testdebug.h"

#include <linux/delay.h>
#include <linux/vmalloc.h>
#include <linux/smp_lock.h>
#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,7) )
#include <linux/completion.h>
#endif

/*************************************************************************************
*************************************************************************************/

typedef struct tagLS {
	struct semaphore	semaphore;
} LS, *PLS;

HLOCK OsLockCreate(void)
{
	PLS		pLock = (PLS)OsAllocate(sizeof(LS));

	if(pLock) {
		sema_init(&pLock->semaphore, 1);
	}

	return (HLOCK)pLock;
}

void OsLockDestroy(HLOCK hLock)
{
	PLS		pLock = (PLS)hLock;

	if(pLock) {
		OsFree(pLock);
	}
}

void OsLockLock(HLOCK hLock)
{
	PLS		pLock = (PLS)hLock;

	down(&pLock->semaphore);
}

void OsLockUnlock(HLOCK hLock)
{
	PLS		pLock = (PLS)hLock;

	up(&pLock->semaphore);
}

/*************************************************************************************
	MEMORY
*************************************************************************************/

int OsMemCmp(PVOID Buff1, PVOID Buff2, UINT32 Count)
{
	return memcmp(Buff1, Buff2, Count);
}

PVOID OsMemCpy(PVOID pDest, PVOID pSrc, UINT32 Count)
{
	return memcpy(pDest, pSrc, Count);
}

void OsMemSet(PVOID Buffer, UINT8 c, UINT32 Count)
{
	memset(Buffer, c, Count);
}

void* OsAllocate(unsigned Size)
{
	PVOID	pMem = vmalloc((unsigned long)Size);

	if(NULL != pMem) {
		OsMemSet(pMem, 0, Size);
//		OsMemSet(pMem, 0xcdcdcdcd, Size);
	}
	else {
		TRACE(T__ERROR, ("%s: could not allocate %d bytes???\n", __FUNCTION__, Size));
	}

	return pMem;
}

void OsFree(void* pMem)
{
	vfree(pMem);
}

/*************************************************************************************
*************************************************************************************/

static spinlock_t atomic_lock __attribute__((unused)) = SPIN_LOCK_UNLOCKED;

/****************************************************************************************
    The OsAtomicCompareAndSwap function compares the value at the specified address with 
	oldVal. The value of newValue is written to the address only if oldValue and the 
	value at the address are equal. OSCompareAndSwap returns true if newValue is written 
	to the address; otherwise, it returns false. 
    
    Params:
        oldValue The value to compare at address.
        newValue The value to write to address if oldValue compares true.
        address The 4-byte aligned address of the data to update atomically.
    Result: true if newValue was written to the address. 
****************************************************************************************/
BOOL OsAtomicCompareAndSwap (PVOID oldValue, PVOID newValue, PVOID* address)
{
#ifdef __HAVE_ARCH_CMPXCHG
	return (cmpxchg(address, oldValue, newValue) == oldValue);
#else
	unsigned long flags;

	spin_lock_irqsave(&atomic_lock, flags);
  	if (*(PUINT32)address ==  (UINT32)oldValue)
  	{
		*(PUINT32)address = (UINT32)newValue;
		spin_unlock_irqrestore(&atomic_lock, flags);
  		return TRUE;
  	}
	spin_unlock_irqrestore(&atomic_lock, flags);

	return FALSE;
#endif
}
/****************************************************************************************
    OsAtomicAdd
   
    OsAtomicAdd function adds the specified amount to the value at the specified 
    address and returns the result.
    Params:
        amount  The amount to add.
        address The 4-byte aligned address of the value to update atomically.
    Result: The result of the addition.
****************************************************************************************/
INT32	OsAtomicAdd   (INT32 amount, PINT32 address)
{
	unsigned long flags;
	atomic_t *v = (atomic_t *)address;

	spin_lock_irqsave(&atomic_lock, flags);
	atomic_add(amount, v);
	amount = atomic_read(v);
	spin_unlock_irqrestore(&atomic_lock, flags);

	return amount;
}

INT32	OsAtomicIncrement (PINT32 address)
{
	unsigned long flags;
	atomic_t *v = (atomic_t *)address;
	INT32 amount;

	spin_lock_irqsave(&atomic_lock, flags);
	atomic_inc(v);
	amount = atomic_read(v);
	spin_unlock_irqrestore(&atomic_lock, flags);

	return amount;
}

INT32	OsAtomicDecrement (PINT32 address)
{
	unsigned long flags;
	atomic_t *v = (atomic_t *)address;
	INT32 amount;

	spin_lock_irqsave(&atomic_lock, flags);
	atomic_dec(v);
	amount = atomic_read(v);
	spin_unlock_irqrestore(&atomic_lock, flags);

	return amount;
}

/*************************************************************************************
*************************************************************************************/

#define TICKS_TO_MSECS(ticks) (ticks*(1000/HZ))

static inline long MSECS_TO_TICKS(UINT32 msecs)
{
	long ticks = (msecs)/(1000/HZ);

	// round up to next tick
	if ((!ticks && msecs) || (msecs % (1000/HZ)) >= ((1000/HZ) / 2))
		ticks++;

	return ticks;
}

typedef struct TIME_OUT_INSTANCE_TYPE {
	struct tq_struct			TaskQueue;
	struct timer_list			Timer;
	UINT32								mSec;
	UINT32								bLocked;
	PFREE_FUNC						pFuncFree;
	PVOID									pRefData;
	PCBFUNC								pTimeOutCallBack;
} TIME_OUT_INSTANCE_T, *PTIME_OUT_INSTANCE_T;

// bit number for delete syncronization
#define TASK_DELETE		1
// bit number indicating that task has been queued
#define TASK_QUEUED		2

static void TimerThreadFunction(void* pData);

static void TimeOutHandler(unsigned long Data)
{
	PTIME_OUT_INSTANCE_T	pTimeOutInstance = (PTIME_OUT_INSTANCE_T)Data;

	if(!test_and_set_bit(TASK_QUEUED, (void*)&pTimeOutInstance->bLocked)) {
		MOD_INC_USE_COUNT;
		if (OsModemThreadSchedule(&pTimeOutInstance->TaskQueue) == 0) {
			MOD_DEC_USE_COUNT;
			clear_bit(TASK_QUEUED, (void*)(&pTimeOutInstance->bLocked));
		}
	}
}

static void TimerThreadFunction(void* pData)
{
	PTIME_OUT_INSTANCE_T	pTimeOutInstance = (PTIME_OUT_INSTANCE_T)pData;

	if(test_bit(TASK_DELETE, (void*)&pTimeOutInstance->bLocked)) {
		clear_bit(TASK_QUEUED, (void*)(&pTimeOutInstance->bLocked));
		MOD_DEC_USE_COUNT;
		return;
	}

	pTimeOutInstance->pTimeOutCallBack(pTimeOutInstance->pRefData);

	if(test_bit(TASK_DELETE, (void*)&pTimeOutInstance->bLocked)) {
		clear_bit(TASK_QUEUED, (void*)(&pTimeOutInstance->bLocked));
		MOD_DEC_USE_COUNT;
		return;
	}

	clear_bit(TASK_QUEUED, (void*)(&pTimeOutInstance->bLocked));
	mod_timer(&(pTimeOutInstance -> Timer), jiffies + MSECS_TO_TICKS(pTimeOutInstance->mSec));

	MOD_DEC_USE_COUNT;
}

static DECLARE_TASK_QUEUE(tq_mdmthrd);
static DECLARE_WAIT_QUEUE_HEAD(mdmthrd_wait);
#ifdef DECLARE_COMPLETION
static DECLARE_COMPLETION(mdmthrd_exited);
#endif
static int mdmthrd_pid;

int OsModemThreadSchedule(PVOID task)
{
#if 0
    return schedule_task(task);
#else
	int ret;

	if(mdmthrd_pid <= 0) {
		printk(KERN_ERR"%s: WARNING: modem thread not present\n", __FUNCTION__);
	}
#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,19) )
	ret = queue_task((struct tq_struct *)task, &tq_mdmthrd);
#else
	{
		struct tq_struct *bh_pointer = (struct tq_struct *)task;
		task_queue *bh_list = &tq_mdmthrd;

		ret = 0;
		if (!test_and_set_bit(0,&bh_pointer->sync)) {
			unsigned long flags;
			spin_lock_irqsave(&tqueue_lock, flags);
			bh_pointer->next = *bh_list;
			*bh_list = bh_pointer;
			spin_unlock_irqrestore(&tqueue_lock, flags);
			ret = 1;
		}
	}
#endif
	wake_up(&mdmthrd_wait);
	return ret;
#endif
}

/* lock_kernel() must be called before this function */
static void thrd_daemonize(void)
{
#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) )
	exit_files(current);  /* daemonize doesn't do exit_files */
	current->files = init_task.files;
	atomic_inc(&current->files->count);
#endif
#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,2,18) ) && !defined(daemonize)
	{
		struct fs_struct *fs;

		exit_mm(current);

		current->session = 1;
		current->pgrp = 1;

		exit_fs(current);	/* current->fs->count--; */
		fs = init_task.fs;
		current->fs = fs;
		atomic_inc(&fs->count);
	}
#else
	daemonize();
#endif

#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,10) ) && \
		( LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0) )
	reparent_to_init();
#endif
}

#ifndef DECLARE_COMPLETION
static int mdmthrd_running;
#endif

static int mdmthrd(void *startup)
{
  	register struct task_struct *curtask = current;

	lock_kernel();

	/*  This thread doesn't need any user-level access,
	 *  so get rid of all our resources.
	 */
	thrd_daemonize();

	strncpy(curtask->comm, __FUNCTION__, sizeof(curtask->comm));

	unlock_kernel();

	spin_lock_irq(&curtask->sigmask_lock);
	sigemptyset(&curtask->blocked);
	flush_signals(curtask);
	{ // flush_signal_handlers
		int i;
		struct k_sigaction *ka = &curtask->sig->action[0];
		for (i = _NSIG ; i != 0 ; i--) {
			if (ka->sa.sa_handler != SIG_IGN)
				ka->sa.sa_handler = SIG_DFL;
			ka->sa.sa_flags = 0;
			sigemptyset(&ka->sa.sa_mask);
			ka++;
		}
	}
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	recalc_sigpending(curtask);
#endif
	spin_unlock_irq(&curtask->sigmask_lock);

#ifdef DECLARE_COMPLETION
	complete((struct completion *)startup);
#else
	mdmthrd_running = 1;
#endif

	while(1) {
	    set_current_state(TASK_RUNNING);
		run_task_queue(&tq_mdmthrd);
		if(signal_pending(curtask))
			break;
		interruptible_sleep_on(&mdmthrd_wait);
	}

	spin_lock_irq(&curtask->sigmask_lock);
	flush_signals(curtask);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	recalc_sigpending(curtask);
#endif
	spin_unlock_irq(&curtask->sigmask_lock);

#ifdef DECLARE_COMPLETION
#if defined(complete_and_exit) || ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,9) )
	complete_and_exit(&mdmthrd_exited, 0);
#else
	complete(&mdmthrd_exited);
	up_and_exit(NULL, 0);
#endif
#else
	mdmthrd_running = 0;
	return 0;
#endif
}

static INT32 ntimers;

HOSTIMER OsCreatePeriodicTimer(	IN UINT32				InitialTimeOut,
																IN PCBFUNC			pTimeOutCallBack,
																IN PALLOC_FUNC	pFuncAlloc,
																IN PFREE_FUNC		pFuncFree,
																IN PVOID				pRefData,
																OUT HTHREAD*		pThreadId)
{
	PTIME_OUT_INSTANCE_T		pTimeOutInstance;

	if(pFuncAlloc == 0) {
		TRACE(T__ERROR, ("OsCreatePeriodicTimer ERROR, pFuncAlloc is NULL???\n"));
		return 0;
	}
	if(pFuncFree == 0) {
		TRACE(T__ERROR, ("OsCreatePeriodicTimer ERROR, pFuncFree is NULL???\n"));
		return 0;
	}
	
	pTimeOutInstance = pFuncAlloc(sizeof(TIME_OUT_INSTANCE_T), pRefData);
	if(0 == pTimeOutInstance) {
		TRACE(T__ERROR, ("OsCreatePeriodicTimer ERROR, pTimeOutInstance is NULL???\n"));
		return 0;
	}

	INIT_TQUEUE(&pTimeOutInstance -> TaskQueue, TimerThreadFunction, pTimeOutInstance);

	pTimeOutInstance->pFuncFree						= pFuncFree;
	pTimeOutInstance->pRefData						= pRefData;
	pTimeOutInstance->pTimeOutCallBack		= pTimeOutCallBack;
	pTimeOutInstance->bLocked							= 0;
	pTimeOutInstance->mSec								= InitialTimeOut;

	init_timer(&pTimeOutInstance->Timer);

	pTimeOutInstance->Timer.function			= TimeOutHandler;
	pTimeOutInstance->Timer.data					= (unsigned long)pTimeOutInstance;
	pTimeOutInstance->Timer.expires				= jiffies + 1;

	if(OsAtomicIncrement(&ntimers) == 1) {
#ifdef COMPLETION_INITIALIZER
		static struct completion startup = COMPLETION_INITIALIZER(startup);
#endif
 
 		MOD_INC_USE_COUNT;
#ifdef COMPLETION_INITIALIZER
		mdmthrd_pid = kernel_thread(mdmthrd, &startup, 0);
		wait_for_completion(&startup);
#else
		mdmthrd_running = 0;
		mdmthrd_pid = kernel_thread(mdmthrd, NULL, 0);
		{
			int i;

			/* wait 5 seconds maximum for modem thread to start */
			for (i=5000; !mdmthrd_running && (i > 0); i -= 50) {
				OsSleep(50);
			}
			if(i <= 0) {
				printk(KERN_ERR"%s: giving up on modem thread\n", __FUNCTION__);
			}
		}
#endif
		if(mdmthrd_pid <= 0) {
			MOD_DEC_USE_COUNT;
			OsAtomicDecrement(&ntimers);
			pTimeOutInstance->pFuncFree(pTimeOutInstance, pTimeOutInstance->pRefData);
			return FALSE;
		}
	}

	add_timer(&(pTimeOutInstance->Timer));
	if(pThreadId) {
		*pThreadId = (HTHREAD)mdmthrd_pid;
	}

	return (HOSTIMER)pTimeOutInstance;
}

void OsDestroyPeriodicTimer(IN HOSTIMER hTimeOut)
{
	PTIME_OUT_INSTANCE_T	pTimeOutInstance = (PTIME_OUT_INSTANCE_T)hTimeOut;

	if(pTimeOutInstance == 0) {
		TRACE(T__ERROR, ("\nOSLIN ERROR: OsDestroyPeriodicTimer: pTimeOutInstance == NULL???\n"));
		return;
	}

	set_bit(TASK_DELETE, (void*)(&pTimeOutInstance->bLocked));

#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0) )
	del_timer_sync(&pTimeOutInstance->Timer);
#else
	del_timer(&pTimeOutInstance->Timer);
#endif

	while(test_bit(TASK_QUEUED, (void*)&pTimeOutInstance->bLocked)) {
	    schedule();
	}

	pTimeOutInstance->pFuncFree(pTimeOutInstance, pTimeOutInstance->pRefData);

	if(OsAtomicDecrement(&ntimers) == 0) {
		if(mdmthrd_pid > 0) {
			int r = kill_proc(mdmthrd_pid, SIGKILL, 1);
			if(r) {
			    printk(KERN_ERR"%s: kill_proc mdmthrd_pid=%d r=%d\n", __FUNCTION__, mdmthrd_pid, r);
			}
#ifdef DECLARE_COMPLETION
			wait_for_completion(&mdmthrd_exited);
#else
			{
				int i;

				/* wait 5 seconds maximum for modem thread to terminate */
				for (i=5000; mdmthrd_running && (i > 0); i -= 50) {
					OsSleep(50);
				}
				if(i <= 0) {
					printk(KERN_ERR"%s: giving up on modem thread\n", __FUNCTION__);
				}
			}
#endif
			MOD_DEC_USE_COUNT;
		}
	}
}

BOOL OsSetPeriodicTimer(IN HOSTIMER hTimeOut, IN UINT32 NewTimeOut)
{
	PTIME_OUT_INSTANCE_T	pTimeOutInstance = (PTIME_OUT_INSTANCE_T)hTimeOut;

	if(pTimeOutInstance == 0) {
		TRACE(T__ERROR, ("\nERROR!!! OsSetPeriodicTimer: pTimeOutInstance == NULL\n"));
		return FALSE;
	}

	if(NewTimeOut) {
		pTimeOutInstance->mSec = NewTimeOut;

#if 0 // because we have got conflict with add_timer in scheduled task	
		del_timer(&pTimeOutInstance->Timer);
		init_timer(&pTimeOutInstance->Timer);
		pTimeOutInstance->Timer.function			= TimeOutHandler;
		pTimeOutInstance->Timer.data					= (unsigned long)pTimeOutInstance;
		pTimeOutInstance->Timer.expires				= jiffies + MSECS_TO_TICKS(pTimeOutInstance->mSec);

		add_timer(&(pTimeOutInstance->Timer));
#endif

		return TRUE;
	}

	return FALSE;
}

void OsImmediateTimeOut(IN HOSTIMER hTimeOut)
{
#if 0
	PTIME_OUT_INSTANCE_T	pTimeOutInstance = (PTIME_OUT_INSTANCE_T)hTimeOut;

	if(pTimeOutInstance == 0) {
		TRACE(T__ERROR, ("\nERROR!!! OsImmediateTimeOut: pTimeOutInstance == NULL\n"));
		return;
	}

	if(pTimeOutInstance->TaskQueue.routine == 0) {
		TRACE(T__ERROR, ("\nERROR!!! OSImmediateTimeOut: TaskQueue.routine == NULL\n"));
		return;
	}

	del_timer(&pTimeOutInstance->Timer);
	schedule_task(&pTimeOutInstance->TaskQueue);
#endif
}

void OsCallOnMyStack(	IN PCBFUNC pFunc,
											IN UINT32 lParam,
											IN PUINT32 pTopOfStack,
											IN UINT32 StackSizeBytes)
{
	if(pFunc)
		pFunc((PVOID)lParam);
}

/*************************************************************************************
*************************************************************************************/


void    OsDebugVPrintf       (const char *szFormat, va_list args)
{
	char printk_buf[1024];

	printk_buf[0]= '\0';
#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,4,10) )
	vsprintf(printk_buf, szFormat, args);
#else
	vsnprintf(printk_buf, sizeof(printk_buf), szFormat, args);
#endif

	printk_buf[sizeof(printk_buf)-1]= '\0';

	printk("%s", printk_buf);
}

void    OsDebugPrintf       (const char *szFormat, ...)
{
	va_list args;

	va_start(args, szFormat);
	OsDebugVPrintf(szFormat, args);
	va_end(args);
}

void    OsDebugBreakpoint   ( LPCSTR szMsg )
{
	OsDebugPrintf("%s: %s\n", __FUNCTION__, szMsg);
	KDB_ENTER();
}

/*************************************************************************************
*************************************************************************************/

HTHREAD OsGetCurrentThread(void)
{
	return (HTHREAD)(in_interrupt() ? 0 : current->pid);
}

/*************************************************************************************
*************************************************************************************/

UINT32 OsGetSystemTime(void)
{
	struct timeval		timestamp;

	do_gettimeofday(&timestamp);

	return (timestamp.tv_sec*1000 + timestamp.tv_usec/1000);
}

/*************************************************************************************
*************************************************************************************/

void OsSleep(UINT32 ms)
{
	if(in_interrupt()) {
		printk(KERN_ERR"%s(%lu): called in interrupt context!\n", __FUNCTION__, ms);
		KDB_ENTER();
		return;
	}

	if(ms > TICKS_TO_MSECS(1)) {
		UINT32 start, end;
		long timeout = MSECS_TO_TICKS(ms);
		//long ort = timeout;

		start = OsGetSystemTime();
		do {
			set_current_state(TASK_UNINTERRUPTIBLE);
		} while ((timeout = schedule_timeout(timeout)));
		end = OsGetSystemTime() - start;
		if(end < ms) {
			//printk(KERN_ERR"OsSleep: short timeout; wanted %ld got %ldms ort=%ld\n", ms, end, ort);
			OsSleep(end);
		}
		//else if(end > ms) {
		//	printk(KERN_ERR"OsSleep: timeout %ldms (%ldms too much) ort=%ld\n", ms, end - ms, ort);
		//}
	} else {
		// short delay
		mdelay(ms);
	}
}
